# pylint: disable=too-many-lines

import os
from datetime import datetime
from datetime import timezone

import pytest
import salt.config


@pytest.fixture
def minion_opts(tmp_path):
    """
    Default minion configuration with relative temporary paths to not
    require root permissions.
    """
    root_dir = tmp_path / "minion"
    opts = salt.config.DEFAULT_MINION_OPTS.copy()
    opts["__role"] = "minion"
    opts["root_dir"] = str(root_dir)
    for name in ("cachedir", "pki_dir", "sock_dir", "conf_dir"):
        dirpath = root_dir / name
        dirpath.mkdir(parents=True)
        opts[name] = str(dirpath)
    opts["log_file"] = "logs/minion.log"
    opts["conf_file"] = os.path.join(opts["conf_dir"], "minion")
    return opts


@pytest.fixture
def master_opts(tmp_path):
    """
    Default master configuration with relative temporary paths to not
    require root permissions.
    """
    root_dir = tmp_path / "master"
    opts = salt.config.master_config(None)
    opts["__role"] = "master"
    opts["root_dir"] = str(root_dir)
    for name in ("cachedir", "pki_dir", "sock_dir", "conf_dir"):
        dirpath = root_dir / name
        dirpath.mkdir(parents=True)
        opts[name] = str(dirpath)
    opts["log_file"] = "logs/master.log"
    opts["conf_file"] = os.path.join(opts["conf_dir"], "master")
    return opts


@pytest.fixture
def syndic_opts(tmp_path):
    """
    Default master configuration with relative temporary paths to not
    require root permissions.
    """
    root_dir = tmp_path / "syndic"
    opts = salt.config.DEFAULT_MINION_OPTS.copy()
    opts["syndic_master"] = "127.0.0.1"
    opts["__role"] = "minion"
    opts["root_dir"] = str(root_dir)
    for name in ("cachedir", "pki_dir", "sock_dir", "conf_dir"):
        dirpath = root_dir / name
        dirpath.mkdir(parents=True)
        opts[name] = str(dirpath)
    opts["log_file"] = "logs/syndic.log"
    opts["conf_file"] = os.path.join(opts["conf_dir"], "syndic")
    return opts


@pytest.fixture
def snap_list():
    return {
        "bare": {
            "name": "bare",
            "version": "1.0",
            "revision": "5",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": ["base"],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "core": {
            "name": "core",
            "version": "16-2.61.2",
            "revision": "16928",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": ["core"],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "core18": {
            "name": "core18",
            "version": "20231027",
            "revision": "2812",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": ["base", "disabled"],
            "classic": False,
            "devmode": False,
            "enabled": False,
            "held": False,
        },
        "core22": {
            "name": "core22",
            "version": "20240111",
            "revision": "1122",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": ["base"],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "gnome-3-28-1804": {
            "name": "gnome-3-28-1804",
            "version": "3.28.0-19-g98f9e67.98f9e67",
            "revision": "198",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": [],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "gnome-3-34-1804": {
            "name": "gnome-3-34-1804",
            "version": "0+git.3556cb3",
            "revision": "93",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": [],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "gtk-common-themes": {
            "name": "gtk-common-themes",
            "version": "0.1-81-g442e511",
            "revision": "1535",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": ["disabled"],
            "classic": False,
            "devmode": False,
            "enabled": False,
            "held": False,
        },
        "hello-world": {
            "name": "hello-world",
            "version": "6.3",
            "revision": "28",
            "channel": "latest/stable",
            "publisher": "canonical**",
            "notes": ["held"],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": True,
        },
        "yubioath-desktop": {
            "name": "yubioath-desktop",
            "version": "5.0.5",
            "revision": "12",
            "channel": "latest/stable",
            "publisher": "yubico-snap-store",
            "notes": [],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "bw": {
            "name": "bw",
            "version": "2024.3.1",
            "revision": "60",
            "channel": None,
            "publisher": "bitwarden**",
            "notes": [],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
        "bitwarden": {
            "name": "bitwarden",
            "version": "2024.4.0",
            "revision": "108",
            "channel": "latest/stable",
            "publisher": "bitwarden**",
            "notes": [],
            "classic": False,
            "devmode": False,
            "enabled": True,
            "held": False,
        },
    }


@pytest.fixture
def snap_refresh_list():
    return {
        "hello-world": {
            "name": "hello-world",
            "version": "6.4",
            "revision": "29",
            "publisher": "canonical**",
            "notes": [],
            "size": "20kB",
        },
        "yubioath-desktop": {
            "name": "yubioath-desktop",
            "version": "5.1.0",
            "revision": "13",
            "publisher": "yubico-snap-store",
            "notes": [],
            "size": "238MB",
        },
    }


@pytest.fixture
def snap_options():
    return {
        "cloud.name": "unknown",
        "refresh": {},
        "seed.loaded": True,
        "system.hostname": "foobar",
        "system.network": {},
        "system.timezone": "UTC",
    }


@pytest.fixture
def snap_services():
    return {
        "yubioath-desktop.ea": {"enabled": True, "running": True, "notes": []},
        "yubioath-desktop.ei": {"enabled": True, "running": False, "notes": []},
        "yubioath-desktop.da": {"enabled": False, "running": True, "notes": []},
        "yubioath-desktop.di": {"enabled": False, "running": False, "notes": []},
        "hello-world.foo": {"enabled": True, "running": True, "notes": []},
    }


@pytest.fixture
def snap_info():
    return {
        "contact": "snaps@canonical.com",
        "commands": ["hello-world.env", "hello-world.evil", "hello-world", "hello-world.sh"],
        "snap-id": "buPKUD3TKqCOgLEjjHx5kSiCpIs5cMuQ",
        "license": "unset",
        "installed": "6.3            (28) 20kB -",
        "name": "hello-world",
        "store-url": "https://snapcraft.io/hello-world",
        "summary": "The 'hello-world' of snaps",
        "publisher": "Canonical**",
        "channels": {
            "latest/candidate": "6.4 2024-02-27 (29) 20kB -",
            "latest/beta": "6.4 2024-02-27 (29) 20kB -",
            "latest/edge": "6.4 2024-02-27 (29) 20kB -",
            "latest/stable": "6.4 2024-02-27 (29) 20kB -",
        },
        "tracking": "latest/stable",
        "refresh-date": "today at 12:15 UTC",
        "description": "This is a simple hello world example.\n",
    }


@pytest.fixture
def snap_info_verbose():
    return {
        "contact": "snaps@canonical.com",
        "commands": ["hello-world.env", "hello-world.evil", "hello-world", "hello-world.sh"],
        "license": "unset",
        "name": "hello-world",
        "store-url": "https://snapcraft.io/hello-world",
        "description": "This is a simple hello world example.\n",
        "snap-id": "buPKUD3TKqCOgLEjjHx5kSiCpIs5cMuQ",
        "links": {"contact": ["mailto:snaps@canonical.com"]},
        "health": {"status": "unknown", "message": "health has not been set"},
        "installed": "6.3            (28) 20kB -",
        "notes": {
            "trymode": False,
            "devmode": False,
            "enabled": True,
            "broken": False,
            "confinement": "strict",
            "private": False,
            "jailmode": False,
            "ignore-validation": False,
        },
        "summary": "The 'hello-world' of snaps",
        "publisher": "Canonical**",
        "channels": {
            "latest/candidate": "6.4 2024-02-27 (29) 20kB -",
            "latest/beta": "6.4 2024-02-27 (29) 20kB -",
            "latest/edge": "6.4 2024-02-27 (29) 20kB -",
            "latest/stable": "6.4 2024-02-27 (29) 20kB -",
        },
        "tracking": "latest/stable",
        "refresh-date": "today at 12:15 UTC",
    }


@pytest.fixture
def snap_info_file():
    return {
        "path": "/root/tmp/bw_60.snap",
        "name": "bw",
        "summary": "Bitwarden CLI - A secure and free password manager for all of your devices.",
        "version": "2024.3.1 -",
        "build-date": "4 days ago, at 14:01 CEST",
        "license": "unset",
        "description": "Bitwarden, Inc. is the parent company of 8bit Solutions LLC.\n\nNAMED BEST PASSWORD MANAGER BY THE VERGE, U.S. NEWS &amp; WORLD REPORT,\nCNET, AND MORE.\n\nManage, store, secure, and share unlimited passwords across unlimited\ndevices from anywhere. Bitwarden delivers open source password management\nsolutions to everyone, whether at  home, at work, or on the go.\n\nGenerate strong, unique, and random passwords based on security\nrequirements for every website you frequent.\n\nBitwarden Send quickly transmits encrypted information --- files and\nplaintext -- directly to anyone.\n\nBitwarden offers Teams and Enterprise plans for companies so you can\nsecurely share passwords with colleagues.\n\nWhy Choose Bitwarden:\n\nWorld-Class Encryption\nPasswords are protected with advanced end-to-end encryption (AES-256 bit,\nsalted hashing, and PBKDF2 SHA-256) so your data stays secure and private.\n\nBuilt-in Password Generator\nGenerate strong, unique, and random passwords based on security\nrequirements for every website you frequent.\n\nGlobal Translations\nBitwarden translations exist in 40 languages and are growing, thanks to our\nglobal community.\n\nCross-Platform Applications\nSecure and share sensitive data within your Bitwarden Vault from any\nbrowser, mobile device, or desktop OS, and more.\n",
        "commands": ["bw"],
        "notes": {"private": False, "confinement": "strict"},
        "base": "core22",
        "sha3-384": "rCj3QCM0PMpsn4FiyOqBpUBtjbU_PLGrTOcr96Q9jTKjV96NSRDpW9lZIpmrsSJU",
    }


@pytest.fixture
def snap_known():
    return [
        {
            "type": "snap-revision",
            "authority-id": "canonical",
            "snap-sha3-384": "AH7zvZLOXzHcp3gxaWTmGUOrmsXYJmACXFiCBoydL-H1PlC9G43rGAJs3WiyzOb_",
            "developer-id": "GKq9csPRQp1E5kUglZG9QTqDEBLrcszO",
            "provenance": "global-upload",
            "snap-id": "JUJH91Ved74jd4ZgJCpzMBtYbPOzTlsD",
            "snap-revision": 142,
            "snap-size": 123101184,
            "timestamp": datetime(2024, 4, 3, 17, 49, 48, 613165, tzinfo=timezone.utc),
            "sign-key-sha3-384": "BWDEoaqyr25nF5SNCvEv2v7QnM9QsfCc0PBMYD_i2NGSQ32EF2d4D0hqUel3m8ul",
        },
        {
            "type": "snap-revision",
            "authority-id": "canonical",
            "snap-sha3-384": "tZ4sZcgQR8QSIdneZKmUj44OXwnVm_64iWwPmwh_3Jfi5eGCRyDYEnfrAcIsX2D_",
            "developer-id": "canonical",
            "provenance": "global-upload",
            "snap-id": "TIM9dBBJEceEjMpwaB3fiuZ3AdSykgDO",
            "snap-revision": 93,
            "snap-size": 228999168,
            "timestamp": datetime(2023, 4, 24, 16, 34, 37, 479211, tzinfo=timezone.utc),
            "sign-key-sha3-384": "BWDEoaqyr25nF5SNCvEv2v7QnM9QsfCc0PBMYD_i2NGSQ32EF2d4D0hqUel3m8ul",
        },
        {
            "type": "snap-revision",
            "authority-id": "canonical",
            "snap-sha3-384": "rCj3QCM0PMpsn4FiyOqBpUBtjbU_PLGrTOcr96Q9jTKjV96NSRDpW9lZIpmrsSJU",
            "developer-id": "SflUpHyJuL9BkjUnFAgINhCW9QjI5tow",
            "provenance": "global-upload",
            "snap-id": "n0XNnJXxSdYYHRYwtfYV2mBSkzH1Fbkn",
            "snap-revision": 60,
            "snap-size": 28114944,
            "timestamp": datetime(2024, 4, 8, 12, 46, 12, 441719, tzinfo=timezone.utc),
            "sign-key-sha3-384": "BWDEoaqyr25nF5SNCvEv2v7QnM9QsfCc0PBMYD_i2NGSQ32EF2d4D0hqUel3m8ul",
        },
        {
            "type": "snap-revision",
            "authority-id": "canonical",
            "snap-sha3-384": "7uNHCak9aSwcICRfP1YmJNGrRqROw45Ihj1aFx7S74MUfXYedKyZQzbjkJC2CCYd",
            "developer-id": "canonical",
            "provenance": "global-upload",
            "snap-id": "amcUKQILKXHHTlmSa7NMdnXSx02dNeeT",
            "snap-revision": 1122,
            "snap-size": 77819904,
            "timestamp": datetime(2024, 1, 11, 16, 31, 37, 728357, tzinfo=timezone.utc),
            "sign-key-sha3-384": "BWDEoaqyr25nF5SNCvEv2v7QnM9QsfCc0PBMYD_i2NGSQ32EF2d4D0hqUel3m8ul",
        },
    ]


@pytest.fixture
def snap_interfaces():
    return {
        "audio-playback": {
            "name": "audio-playback",
            "summary": "allows audio playback via supporting services",
            "plugs": [{"snap": "bitwarden", "plug": "audio-playback"}],
            "slots": [{"snap": "core", "slot": "audio-playback"}],
        },
        "browser-support": {
            "name": "browser-support",
            "summary": "allows access to various APIs needed by modern web browsers",
            "plugs": [{"snap": "bitwarden", "plug": "browser-support"}],
            "slots": [{"snap": "core", "slot": "browser-support"}],
        },
        "content": {
            "name": "content",
            "summary": "allows sharing code and data with other snaps",
            "plugs": [
                {
                    "snap": "bitwarden",
                    "plug": "gnome-3-28-1804",
                    "attrs": {
                        "content": "gnome-3-28-1804",
                        "default-provider": "gnome-3-28-1804",
                        "target": "$SNAP/gnome-platform",
                    },
                },
                {
                    "snap": "bitwarden",
                    "plug": "gtk-3-themes",
                    "attrs": {
                        "content": "gtk-3-themes",
                        "default-provider": "gtk-common-themes",
                        "target": "$SNAP/data-dir/themes",
                    },
                },
                {
                    "snap": "bitwarden",
                    "plug": "icon-themes",
                    "attrs": {
                        "content": "icon-themes",
                        "default-provider": "gtk-common-themes",
                        "target": "$SNAP/data-dir/icons",
                    },
                },
                {
                    "snap": "bitwarden",
                    "plug": "sound-themes",
                    "attrs": {
                        "content": "sound-themes",
                        "default-provider": "gtk-common-themes",
                        "target": "$SNAP/data-dir/sounds",
                    },
                },
            ],
            "slots": [
                {
                    "snap": "gnome-3-28-1804",
                    "slot": "gnome-3-28-1804",
                    "attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                },
                {
                    "snap": "gnome-3-34-1804",
                    "slot": "gnome-3-34-1804",
                    "attrs": {"content": "gnome-3-34-1804", "read": ["/"]},
                },
            ],
        },
        "desktop": {
            "name": "desktop",
            "summary": "allows access to basic graphical desktop resources",
            "plugs": [{"snap": "bitwarden", "plug": "desktop"}],
            "slots": [{"snap": "core", "slot": "desktop"}],
        },
        "desktop-legacy": {
            "name": "desktop-legacy",
            "summary": "allows privileged access to desktop legacy methods",
            "plugs": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            "slots": [{"snap": "core", "slot": "desktop-legacy"}],
        },
        "gsettings": {
            "name": "gsettings",
            "summary": "allows access to any gsettings item of current user",
            "plugs": [{"snap": "bitwarden", "plug": "gsettings"}],
            "slots": [{"snap": "core", "slot": "gsettings"}],
        },
        "home": {
            "name": "home",
            "summary": "allows access to non-hidden files in the home directory",
            "plugs": [{"snap": "bitwarden", "plug": "home"}],
            "slots": [{"snap": "core", "slot": "home"}],
        },
        "network": {
            "name": "network",
            "summary": "allows access to the network",
            "plugs": [{"snap": "bitwarden", "plug": "network"}, {"snap": "bw", "plug": "network"}],
            "slots": [{"snap": "core", "slot": "network"}],
        },
        "network-bind": {
            "name": "network-bind",
            "summary": "allows operating as a network service",
            "plugs": [{"snap": "bw", "plug": "network-bind"}],
            "slots": [{"snap": "core", "slot": "network-bind"}],
        },
        "opengl": {
            "name": "opengl",
            "summary": "allows access to OpenGL stack",
            "plugs": [{"snap": "bitwarden", "plug": "opengl"}],
            "slots": [{"snap": "core", "slot": "opengl"}],
        },
        "pulseaudio": {
            "name": "pulseaudio",
            "summary": "allows operating as or interacting with the pulseaudio service",
            "plugs": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            "slots": [{"snap": "core", "slot": "pulseaudio"}],
        },
        "unity7": {
            "name": "unity7",
            "summary": "allows interacting with Unity 7 services",
            "plugs": [{"snap": "bitwarden", "plug": "unity7"}],
            "slots": [{"snap": "core", "slot": "unity7"}],
        },
        "wayland": {
            "name": "wayland",
            "summary": "allows access to compositors supporting wayland protocol",
            "plugs": [{"snap": "bitwarden", "plug": "wayland"}],
            "slots": [{"snap": "core", "slot": "wayland"}],
        },
        "x11": {
            "name": "x11",
            "summary": "allows interacting with or running as an X11 server",
            "plugs": [{"snap": "bitwarden", "plug": "x11"}],
            "slots": [{"snap": "core", "slot": "x11"}],
        },
    }


@pytest.fixture
def snap_interfaces_all():
    return {
        "account-control": {
            "name": "account-control",
            "summary": "allows managing non-system user accounts",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "account-control"}],
        },
        "accounts-service": {
            "name": "accounts-service",
            "summary": "allows communication with the Accounts service like GNOME Online Accounts",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "accounts-service"}],
        },
        "acrn-support": {
            "name": "acrn-support",
            "summary": "allows operating managing the ACRN hypervisor",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "acrn-support"}],
        },
        "adb-support": {
            "name": "adb-support",
            "summary": "allows operating as Android Debug Bridge service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "adb-support"}],
        },
        "allegro-vcu": {
            "name": "allegro-vcu",
            "summary": "allows access to Allegro Video Code Unit",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "allegro-vcu"}],
        },
        "alsa": {
            "name": "alsa",
            "summary": "allows access to raw ALSA devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "alsa"}],
        },
        "appstream-metadata": {
            "name": "appstream-metadata",
            "summary": "allows access to AppStream metadata",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "appstream-metadata"}],
        },
        "audio-playback": {
            "name": "audio-playback",
            "summary": "allows audio playback via supporting services",
            "plugs": [{"snap": "bitwarden", "plug": "audio-playback"}],
            "slots": [{"snap": "core", "slot": "audio-playback"}],
        },
        "audio-record": {
            "name": "audio-record",
            "summary": "allows audio recording via supporting services",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "audio-record"}],
        },
        "autopilot-introspection": {
            "name": "autopilot-introspection",
            "summary": "allows introspection of application user interface",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "autopilot-introspection"}],
        },
        "avahi-control": {
            "name": "avahi-control",
            "summary": "allows control over service discovery on a local network via the mDNS/DNS-SD protocol suite",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "avahi-control"}],
        },
        "avahi-observe": {
            "name": "avahi-observe",
            "summary": "allows discovery on a local network via the mDNS/DNS-SD protocol suite",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "avahi-observe"}],
        },
        "block-devices": {
            "name": "block-devices",
            "summary": "allows access to disk block devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "block-devices"}],
        },
        "bluetooth-control": {
            "name": "bluetooth-control",
            "summary": "allows managing the kernel bluetooth stack",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "bluetooth-control"}],
        },
        "bluez": {
            "name": "bluez",
            "summary": "allows operating as the bluez service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "bluez"}],
        },
        "bool-file": {
            "name": "bool-file",
            "summary": "allows access to specific file with bool semantics",
            "plugs": [],
            "slots": [],
        },
        "broadcom-asic-control": {
            "name": "broadcom-asic-control",
            "summary": "allows using the broadcom-asic kernel module",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "broadcom-asic-control"}],
        },
        "browser-support": {
            "name": "browser-support",
            "summary": "allows access to various APIs needed by modern web browsers",
            "plugs": [{"snap": "bitwarden", "plug": "browser-support"}],
            "slots": [{"snap": "core", "slot": "browser-support"}],
        },
        "calendar-service": {
            "name": "calendar-service",
            "summary": "allows communication with Evolution Data Service Calendar",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "calendar-service"}],
        },
        "camera": {
            "name": "camera",
            "summary": "allows access to all cameras",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "camera"}],
        },
        "can-bus": {
            "name": "can-bus",
            "summary": "allows access to the CAN bus",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "can-bus"}],
        },
        "cifs-mount": {
            "name": "cifs-mount",
            "summary": "allows mounting and unmounting CIFS filesystems",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "cifs-mount"}],
        },
        "classic-support": {
            "name": "classic-support",
            "summary": "special permissions for the classic snap",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "classic-support"}],
        },
        "contacts-service": {
            "name": "contacts-service",
            "summary": "allows communication with Evolution Data Service Address Book",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "contacts-service"}],
        },
        "content": {
            "name": "content",
            "summary": "allows sharing code and data with other snaps",
            "plugs": [
                {
                    "snap": "bitwarden",
                    "plug": "gnome-3-28-1804",
                    "attrs": {
                        "content": "gnome-3-28-1804",
                        "default-provider": "gnome-3-28-1804",
                        "target": "$SNAP/gnome-platform",
                    },
                },
                {
                    "snap": "bitwarden",
                    "plug": "gtk-3-themes",
                    "attrs": {
                        "content": "gtk-3-themes",
                        "default-provider": "gtk-common-themes",
                        "target": "$SNAP/data-dir/themes",
                    },
                },
                {
                    "snap": "bitwarden",
                    "plug": "icon-themes",
                    "attrs": {
                        "content": "icon-themes",
                        "default-provider": "gtk-common-themes",
                        "target": "$SNAP/data-dir/icons",
                    },
                },
                {
                    "snap": "bitwarden",
                    "plug": "sound-themes",
                    "attrs": {
                        "content": "sound-themes",
                        "default-provider": "gtk-common-themes",
                        "target": "$SNAP/data-dir/sounds",
                    },
                },
            ],
            "slots": [
                {
                    "snap": "gnome-3-28-1804",
                    "slot": "gnome-3-28-1804",
                    "attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                },
                {
                    "snap": "gnome-3-34-1804",
                    "slot": "gnome-3-34-1804",
                    "attrs": {"content": "gnome-3-34-1804", "read": ["/"]},
                },
            ],
        },
        "core-support": {
            "name": "core-support",
            "summary": "special permissions for the core snap",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "core-support"}],
        },
        "cpu-control": {
            "name": "cpu-control",
            "summary": "allows setting CPU tunables",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "cpu-control"}],
        },
        "cups": {
            "name": "cups",
            "summary": "allows access to the CUPS socket for printing",
            "plugs": [],
            "slots": [],
        },
        "cups-control": {
            "name": "cups-control",
            "summary": "allows access to the CUPS control socket",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "cups-control"}],
        },
        "custom-device": {
            "name": "custom-device",
            "summary": "provides access to custom devices specified via the gadget snap",
            "plugs": [],
            "slots": [],
        },
        "daemon-notify": {
            "name": "daemon-notify",
            "summary": "allows sending daemon status changes to service manager",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "daemon-notify"}],
        },
        "dbus": {
            "name": "dbus",
            "summary": "allows owning a specific name on DBus",
            "plugs": [],
            "slots": [],
        },
        "dcdbas-control": {
            "name": "dcdbas-control",
            "summary": "allows access to Dell Systems Management Base Driver",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "dcdbas-control"}],
        },
        "desktop": {
            "name": "desktop",
            "summary": "allows access to basic graphical desktop resources",
            "plugs": [{"snap": "bitwarden", "plug": "desktop"}],
            "slots": [{"snap": "core", "slot": "desktop"}],
        },
        "desktop-launch": {
            "name": "desktop-launch",
            "summary": "allows snaps to identify and launch desktop applications in (or from) other snaps",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "desktop-launch"}],
        },
        "desktop-legacy": {
            "name": "desktop-legacy",
            "summary": "allows privileged access to desktop legacy methods",
            "plugs": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            "slots": [{"snap": "core", "slot": "desktop-legacy"}],
        },
        "device-buttons": {
            "name": "device-buttons",
            "summary": "allows access to device buttons as input events",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "device-buttons"}],
        },
        "display-control": {
            "name": "display-control",
            "summary": "allows configuring display parameters",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "display-control"}],
        },
        "dm-crypt": {
            "name": "dm-crypt",
            "summary": "allows encryption and decryption of block storage devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "dm-crypt"}],
        },
        "docker": {
            "name": "docker",
            "summary": "allows access to Docker socket",
            "plugs": [],
            "slots": [],
        },
        "docker-support": {
            "name": "docker-support",
            "summary": "allows operating as the Docker daemon",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "docker-support"}],
        },
        "dsp": {
            "name": "dsp",
            "summary": "allows controlling digital signal processors on certain boards",
            "plugs": [],
            "slots": [],
        },
        "dvb": {
            "name": "dvb",
            "summary": "allows access to all DVB (Digital Video Broadcasting) devices and APIs",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "dvb"}],
        },
        "empty": {
            "name": "empty",
            "summary": "allows testing without providing any additional permissions",
            "plugs": [],
            "slots": [],
        },
        "firewall-control": {
            "name": "firewall-control",
            "summary": "allows control over network firewall",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "firewall-control"}],
        },
        "fpga": {
            "name": "fpga",
            "summary": "allows access to the FPGA subsystem",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "fpga"}],
        },
        "framebuffer": {
            "name": "framebuffer",
            "summary": "allows access to universal framebuffer devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "framebuffer"}],
        },
        "fuse-support": {
            "name": "fuse-support",
            "summary": "allows access to the FUSE file system",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "fuse-support"}],
        },
        "fwupd": {
            "name": "fwupd",
            "summary": "allows operating as the fwupd service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "fwupd"}],
        },
        "gconf": {
            "name": "gconf",
            "summary": "allows access to any item from the legacy gconf configuration system for the current user",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "gconf"}],
        },
        "gpg-keys": {
            "name": "gpg-keys",
            "summary": "allows reading gpg user configuration and keys and updating gpg's random seed file",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "gpg-keys"}],
        },
        "gpg-public-keys": {
            "name": "gpg-public-keys",
            "summary": "allows reading gpg public keys and non-sensitive configuration",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "gpg-public-keys"}],
        },
        "gpio": {
            "name": "gpio",
            "summary": "allows access to specific GPIO pin",
            "plugs": [],
            "slots": [],
        },
        "gpio-control": {
            "name": "gpio-control",
            "summary": "allows control of all aspects of GPIO pins",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "gpio-control"}],
        },
        "gpio-memory-control": {
            "name": "gpio-memory-control",
            "summary": "allows write access to all gpio memory",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "gpio-memory-control"}],
        },
        "greengrass-support": {
            "name": "greengrass-support",
            "summary": "allows operating as the Greengrass service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "greengrass-support"}],
        },
        "gsettings": {
            "name": "gsettings",
            "summary": "allows access to any gsettings item of current user",
            "plugs": [{"snap": "bitwarden", "plug": "gsettings"}],
            "slots": [{"snap": "core", "slot": "gsettings"}],
        },
        "hardware-observe": {
            "name": "hardware-observe",
            "summary": "allows reading information about system hardware",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "hardware-observe"}],
        },
        "hardware-random-control": {
            "name": "hardware-random-control",
            "summary": "allows control over the hardware random number generator",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "hardware-random-control"}],
        },
        "hardware-random-observe": {
            "name": "hardware-random-observe",
            "summary": "allows reading from hardware random number generator",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "hardware-random-observe"}],
        },
        "hidraw": {
            "name": "hidraw",
            "summary": "allows access to specific hidraw device",
            "plugs": [],
            "slots": [],
        },
        "home": {
            "name": "home",
            "summary": "allows access to non-hidden files in the home directory",
            "plugs": [{"snap": "bitwarden", "plug": "home"}],
            "slots": [{"snap": "core", "slot": "home"}],
        },
        "hostname-control": {
            "name": "hostname-control",
            "summary": "allows configuring the system hostname",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "hostname-control"}],
        },
        "hugepages-control": {
            "name": "hugepages-control",
            "summary": "allows controlling hugepages",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "hugepages-control"}],
        },
        "i2c": {
            "name": "i2c",
            "summary": "allows access to specific I2C controller",
            "plugs": [],
            "slots": [],
        },
        "iio": {
            "name": "iio",
            "summary": "allows access to a specific IIO device",
            "plugs": [],
            "slots": [],
        },
        "intel-mei": {
            "name": "intel-mei",
            "summary": "allows access to the Intel MEI management interface",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "intel-mei"}],
        },
        "io-ports-control": {
            "name": "io-ports-control",
            "summary": "allows access to all I/O ports",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "io-ports-control"}],
        },
        "ion-memory-control": {
            "name": "ion-memory-control",
            "summary": "allows access to The Android ION memory allocator",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "ion-memory-control"}],
        },
        "jack1": {
            "name": "jack1",
            "summary": "allows interacting with a JACK1 server",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "jack1"}],
        },
        "joystick": {
            "name": "joystick",
            "summary": "allows access to joystick devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "joystick"}],
        },
        "juju-client-observe": {
            "name": "juju-client-observe",
            "summary": "allows read access to juju client configuration",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "juju-client-observe"}],
        },
        "kernel-crypto-api": {
            "name": "kernel-crypto-api",
            "summary": "allows access to the Linux kernel crypto API",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "kernel-crypto-api"}],
        },
        "kernel-module-control": {
            "name": "kernel-module-control",
            "summary": "allows insertion, removal and querying of kernel modules",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "kernel-module-control"}],
        },
        "kernel-module-load": {
            "name": "kernel-module-load",
            "summary": "allows constrained control over kernel module loading",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "kernel-module-load"}],
        },
        "kernel-module-observe": {
            "name": "kernel-module-observe",
            "summary": "allows querying of kernel modules",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "kernel-module-observe"}],
        },
        "kubernetes-support": {
            "name": "kubernetes-support",
            "summary": "allows operating as the Kubernetes service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "kubernetes-support"}],
        },
        "kvm": {
            "name": "kvm",
            "summary": "allows access to the kvm device",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "kvm"}],
        },
        "libvirt": {
            "name": "libvirt",
            "summary": "allows access to libvirt service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "libvirt"}],
        },
        "locale-control": {
            "name": "locale-control",
            "summary": "allows control over system locale",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "locale-control"}],
        },
        "location-control": {
            "name": "location-control",
            "summary": "allows operating as the location service",
            "plugs": [],
            "slots": [],
        },
        "location-observe": {
            "name": "location-observe",
            "summary": "allows access to the current physical location",
            "plugs": [],
            "slots": [],
        },
        "log-observe": {
            "name": "log-observe",
            "summary": "allows read access to system logs",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "log-observe"}],
        },
        "login-session-control": {
            "name": "login-session-control",
            "summary": "allows setup of login session & seat",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "login-session-control"}],
        },
        "login-session-observe": {
            "name": "login-session-observe",
            "summary": "allows reading login and session information",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "login-session-observe"}],
        },
        "lxd": {
            "name": "lxd",
            "summary": "allows access to the LXD socket",
            "plugs": [],
            "slots": [],
        },
        "lxd-support": {
            "name": "lxd-support",
            "summary": "allows operating as the LXD service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "lxd-support"}],
        },
        "maliit": {
            "name": "maliit",
            "summary": "allows operating as the Maliit service",
            "plugs": [],
            "slots": [],
        },
        "media-control": {
            "name": "media-control",
            "summary": "allows access to media control devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "media-control"}],
        },
        "media-hub": {
            "name": "media-hub",
            "summary": "allows operating as the media-hub service",
            "plugs": [],
            "slots": [],
        },
        "microceph": {
            "name": "microceph",
            "summary": "allows access to the MicroCeph socket",
            "plugs": [],
            "slots": [],
        },
        "microovn": {
            "name": "microovn",
            "summary": "allows access to the MicroOVN socket",
            "plugs": [],
            "slots": [],
        },
        "microstack-support": {
            "name": "microstack-support",
            "summary": "allows operating as the MicroStack service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "microstack-support"}],
        },
        "mir": {
            "name": "mir",
            "summary": "allows operating as the Mir server",
            "plugs": [],
            "slots": [],
        },
        "modem-manager": {
            "name": "modem-manager",
            "summary": "allows operating as the ModemManager service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "modem-manager"}],
        },
        "mount-control": {
            "name": "mount-control",
            "summary": "allows creating transient and persistent mounts",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "mount-control"}],
        },
        "mount-observe": {
            "name": "mount-observe",
            "summary": "allows reading mount table and quota information",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "mount-observe"}],
        },
        "mpris": {
            "name": "mpris",
            "summary": "allows operating as an MPRIS player",
            "plugs": [],
            "slots": [],
        },
        "multipass-support": {
            "name": "multipass-support",
            "summary": "allows operating as the Multipass service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "multipass-support"}],
        },
        "netlink-audit": {
            "name": "netlink-audit",
            "summary": "allows access to kernel audit system through netlink",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "netlink-audit"}],
        },
        "netlink-connector": {
            "name": "netlink-connector",
            "summary": "allows communication through the kernel netlink connector",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "netlink-connector"}],
        },
        "netlink-driver": {
            "name": "netlink-driver",
            "summary": "allows operating a kernel driver module exposing itself via a netlink protocol family",
            "plugs": [],
            "slots": [],
        },
        "network": {
            "name": "network",
            "summary": "allows access to the network",
            "plugs": [{"snap": "bitwarden", "plug": "network"}, {"snap": "bw", "plug": "network"}],
            "slots": [{"snap": "core", "slot": "network"}],
        },
        "network-bind": {
            "name": "network-bind",
            "summary": "allows operating as a network service",
            "plugs": [{"snap": "bw", "plug": "network-bind"}],
            "slots": [{"snap": "core", "slot": "network-bind"}],
        },
        "network-control": {
            "name": "network-control",
            "summary": "allows configuring networking and network namespaces",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-control"}],
        },
        "network-manager": {
            "name": "network-manager",
            "summary": "allows operating as the NetworkManager service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-manager"}],
        },
        "network-manager-observe": {
            "name": "network-manager-observe",
            "summary": "allows observing NetworkManager settings",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-manager-observe"}],
        },
        "network-observe": {
            "name": "network-observe",
            "summary": "allows querying network status",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-observe"}],
        },
        "network-setup-control": {
            "name": "network-setup-control",
            "summary": "allows access to netplan configuration",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-setup-control"}],
        },
        "network-setup-observe": {
            "name": "network-setup-observe",
            "summary": "allows read access to netplan configuration",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-setup-observe"}],
        },
        "network-status": {
            "name": "network-status",
            "summary": "allows access to network connectivity status",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "network-status"}],
        },
        "nvidia-drivers-support": {
            "name": "nvidia-drivers-support",
            "summary": "NVIDIA drivers userspace system setup support",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "nvidia-drivers-support"}],
        },
        "ofono": {
            "name": "ofono",
            "summary": "allows operating as the ofono service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "ofono"}],
        },
        "online-accounts-service": {
            "name": "online-accounts-service",
            "summary": "allows operating as the Online Accounts service",
            "plugs": [],
            "slots": [],
        },
        "opengl": {
            "name": "opengl",
            "summary": "allows access to OpenGL stack",
            "plugs": [{"snap": "bitwarden", "plug": "opengl"}],
            "slots": [{"snap": "core", "slot": "opengl"}],
        },
        "openvswitch": {
            "name": "openvswitch",
            "summary": "allows access to openvswitch management sockets",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "openvswitch"}],
        },
        "openvswitch-support": {
            "name": "openvswitch-support",
            "summary": "allows operating as the openvswitch service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "openvswitch-support"}],
        },
        "optical-drive": {
            "name": "optical-drive",
            "summary": "allows access to optical drives",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "optical-drive"}],
        },
        "packagekit-control": {
            "name": "packagekit-control",
            "summary": "allows control of the PackageKit service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "packagekit-control"}],
        },
        "password-manager-service": {
            "name": "password-manager-service",
            "summary": "allows access to common password manager services",
            "plugs": [{"snap": "bitwarden", "plug": "password-manager-service"}],
            "slots": [{"snap": "core", "slot": "password-manager-service"}],
        },
        "pcscd": {
            "name": "pcscd",
            "summary": "allows interacting with PCSD daemon\n(e.g. for the PS/SC API library).",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "pcscd"}],
        },
        "personal-files": {
            "name": "personal-files",
            "summary": "allows access to personal files or directories",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "personal-files"}],
        },
        "physical-memory-control": {
            "name": "physical-memory-control",
            "summary": "allows write access to all physical memory",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "physical-memory-control"}],
        },
        "physical-memory-observe": {
            "name": "physical-memory-observe",
            "summary": "allows read access to all physical memory",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "physical-memory-observe"}],
        },
        "pkcs11": {
            "name": "pkcs11",
            "summary": "allows use of pkcs11 framework and access to exposed tokens",
            "plugs": [],
            "slots": [],
        },
        "polkit": {
            "name": "polkit",
            "summary": "allows access to polkitd to check authorisation",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "polkit"}],
        },
        "polkit-agent": {
            "name": "polkit-agent",
            "summary": "allows operation as a polkit agent",
            "plugs": [],
            "slots": [],
        },
        "posix-mq": {
            "name": "posix-mq",
            "summary": "allows access to POSIX message queues",
            "plugs": [],
            "slots": [],
        },
        "power-control": {
            "name": "power-control",
            "summary": "allows setting system power settings",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "power-control"}],
        },
        "ppp": {
            "name": "ppp",
            "summary": "allows operating as the ppp service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "ppp"}],
        },
        "process-control": {
            "name": "process-control",
            "summary": "allows controlling other processes",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "process-control"}],
        },
        "ptp": {
            "name": "ptp",
            "summary": "allows access to the PTP Hardware Clock subsystem",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "ptp"}],
        },
        "pulseaudio": {
            "name": "pulseaudio",
            "summary": "allows operating as or interacting with the pulseaudio service",
            "plugs": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            "slots": [{"snap": "core", "slot": "pulseaudio"}],
        },
        "pwm": {
            "name": "pwm",
            "summary": "allows access to specific PWM channel",
            "plugs": [],
            "slots": [],
        },
        "qualcomm-ipc-router": {
            "name": "qualcomm-ipc-router",
            "summary": "allows access to the Qualcomm IPC Router sockets",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "qualcomm-ipc-router"}],
        },
        "raw-input": {
            "name": "raw-input",
            "summary": "allows access to raw input devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "raw-input"}],
        },
        "raw-usb": {
            "name": "raw-usb",
            "summary": "allows raw access to all USB devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "raw-usb"}],
        },
        "raw-volume": {
            "name": "raw-volume",
            "summary": "allows read/write access to specific disk partition",
            "plugs": [],
            "slots": [],
        },
        "removable-media": {
            "name": "removable-media",
            "summary": "allows access to mounted removable storage",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "removable-media"}],
        },
        "screen-inhibit-control": {
            "name": "screen-inhibit-control",
            "summary": "allows inhibiting the screen saver",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "screen-inhibit-control"}],
        },
        "screencast-legacy": {
            "name": "screencast-legacy",
            "summary": "allows screen recording and audio recording, and also writing to arbitrary filesystem paths",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "screencast-legacy"}],
        },
        "scsi-generic": {
            "name": "scsi-generic",
            "summary": "allows access to SCSI generic driver devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "scsi-generic"}],
        },
        "sd-control": {
            "name": "sd-control",
            "summary": "allows controlling SD cards on certain boards",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "sd-control"}],
        },
        "serial-port": {
            "name": "serial-port",
            "summary": "allows accessing a specific serial port",
            "plugs": [],
            "slots": [],
        },
        "shared-memory": {
            "name": "shared-memory",
            "summary": "allows two snaps to use predefined shared memory objects",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "shared-memory"}],
        },
        "shutdown": {
            "name": "shutdown",
            "summary": "allows shutting down or rebooting the system",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "shutdown"}],
        },
        "snap-refresh-control": {
            "name": "snap-refresh-control",
            "summary": "allows extended control via snapctl over refreshes involving the snap",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "snap-refresh-control"}],
        },
        "snap-themes-control": {
            "name": "snap-themes-control",
            "summary": "allows use of snapd's theme installation API",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "snap-themes-control"}],
        },
        "snapd-control": {
            "name": "snapd-control",
            "summary": "allows communicating with snapd",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "snapd-control"}],
        },
        "spi": {
            "name": "spi",
            "summary": "allows access to specific spi controller",
            "plugs": [],
            "slots": [],
        },
        "ssh-keys": {
            "name": "ssh-keys",
            "summary": "allows reading ssh user configuration and keys",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "ssh-keys"}],
        },
        "ssh-public-keys": {
            "name": "ssh-public-keys",
            "summary": "allows reading ssh public keys and non-sensitive configuration",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "ssh-public-keys"}],
        },
        "steam-support": {
            "name": "steam-support",
            "summary": "allow Steam to configure pressure-vessel containers",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "steam-support"}],
        },
        "storage-framework-service": {
            "name": "storage-framework-service",
            "summary": "allows operating as or interacting with the Storage Framework",
            "plugs": [],
            "slots": [],
        },
        "system-backup": {
            "name": "system-backup",
            "summary": "allows read-only access to the entire system for backups",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "system-backup"}],
        },
        "system-files": {
            "name": "system-files",
            "summary": "allows access to system files or directories",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "system-files"}],
        },
        "system-observe": {
            "name": "system-observe",
            "summary": "allows observing all processes and drivers",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "system-observe"}],
        },
        "system-packages-doc": {
            "name": "system-packages-doc",
            "summary": "allows access to documentation of system packages",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "system-packages-doc"}],
        },
        "system-source-code": {
            "name": "system-source-code",
            "summary": "allows read-only access to /usr/src on the system",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "system-source-code"}],
        },
        "system-trace": {
            "name": "system-trace",
            "summary": "allows using kernel tracing facilities",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "system-trace"}],
        },
        "tee": {
            "name": "tee",
            "summary": "allows access to Trusted Execution Environment devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "tee"}],
        },
        "thumbnailer-service": {
            "name": "thumbnailer-service",
            "summary": "allows operating as or interacting with the Thumbnailer service",
            "plugs": [],
            "slots": [],
        },
        "time-control": {
            "name": "time-control",
            "summary": "allows setting system date and time",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "time-control"}],
        },
        "timeserver-control": {
            "name": "timeserver-control",
            "summary": "allows setting system time synchronization servers",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "timeserver-control"}],
        },
        "timezone-control": {
            "name": "timezone-control",
            "summary": "allows setting system timezone",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "timezone-control"}],
        },
        "tpm": {
            "name": "tpm",
            "summary": "allows access to the Trusted Platform Module device",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "tpm"}],
        },
        "u2f-devices": {
            "name": "u2f-devices",
            "summary": "allows access to u2f devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "u2f-devices"}],
        },
        "ubuntu-download-manager": {
            "name": "ubuntu-download-manager",
            "summary": "allows operating as or interacting with the Ubuntu download manager",
            "plugs": [],
            "slots": [],
        },
        "udisks2": {
            "name": "udisks2",
            "summary": "allows operating as or interacting with the UDisks2 service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "udisks2"}],
        },
        "uhid": {
            "name": "uhid",
            "summary": "allows control over UHID devices",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "uhid"}],
        },
        "uinput": {
            "name": "uinput",
            "summary": "allows access to the uinput device",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "uinput"}],
        },
        "uio": {
            "name": "uio",
            "summary": "allows access to specific uio device",
            "plugs": [],
            "slots": [],
        },
        "unity7": {
            "name": "unity7",
            "summary": "allows interacting with Unity 7 services",
            "plugs": [{"snap": "bitwarden", "plug": "unity7"}],
            "slots": [{"snap": "core", "slot": "unity7"}],
        },
        "unity8": {
            "name": "unity8",
            "summary": "allows operating as or interacting with Unity 8",
            "plugs": [],
            "slots": [],
        },
        "unity8-calendar": {
            "name": "unity8-calendar",
            "summary": "allows operating as or interacting with the Unity 8 Calendar Service",
            "plugs": [],
            "slots": [],
        },
        "unity8-contacts": {
            "name": "unity8-contacts",
            "summary": "allows operating as or interacting with the Unity 8 Contacts Service",
            "plugs": [],
            "slots": [],
        },
        "upower-observe": {
            "name": "upower-observe",
            "summary": "allows operating as or reading from the UPower service",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "upower-observe"}],
        },
        "userns": {
            "name": "userns",
            "summary": "allows the ability to use user namespaces",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "userns"}],
        },
        "vcio": {
            "name": "vcio",
            "summary": "allows access to VideoCore I/O",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "vcio"}],
        },
        "wayland": {
            "name": "wayland",
            "summary": "allows access to compositors supporting wayland protocol",
            "plugs": [{"snap": "bitwarden", "plug": "wayland"}],
            "slots": [{"snap": "core", "slot": "wayland"}],
        },
        "x11": {
            "name": "x11",
            "summary": "allows interacting with or running as an X11 server",
            "plugs": [{"snap": "bitwarden", "plug": "x11"}],
            "slots": [{"snap": "core", "slot": "x11"}],
        },
        "xilinx-dma": {
            "name": "xilinx-dma",
            "summary": "allows access to Xilinx DMA IP on a connected PCIe card",
            "plugs": [],
            "slots": [{"snap": "core", "slot": "xilinx-dma"}],
        },
    }


@pytest.fixture
def snap_connections():
    return {
        "established": [
            {
                "slot": {"snap": "core", "slot": "audio-playback"},
                "plug": {"snap": "bitwarden", "plug": "audio-playback"},
                "interface": "audio-playback",
            },
            {
                "slot": {"snap": "core", "slot": "browser-support"},
                "plug": {"snap": "bitwarden", "plug": "browser-support"},
                "interface": "browser-support",
            },
            {
                "slot": {"snap": "core", "slot": "desktop"},
                "plug": {"snap": "bitwarden", "plug": "desktop"},
                "interface": "desktop",
            },
            {
                "slot": {"snap": "core", "slot": "desktop-legacy"},
                "plug": {"snap": "bitwarden", "plug": "desktop-legacy"},
                "interface": "desktop-legacy",
            },
            {
                "slot": {"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"},
                "plug": {"snap": "bitwarden", "plug": "gnome-3-28-1804"},
                "interface": "content",
                "slot-attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                "plug-attrs": {
                    "content": "gnome-3-28-1804",
                    "default-provider": "gnome-3-28-1804",
                    "target": "$SNAP/gnome-platform",
                },
            },
            {
                "slot": {"snap": "core", "slot": "gsettings"},
                "plug": {"snap": "bitwarden", "plug": "gsettings"},
                "interface": "gsettings",
            },
            {
                "slot": {"snap": "core", "slot": "home"},
                "plug": {"snap": "bitwarden", "plug": "home"},
                "interface": "home",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bitwarden", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "opengl"},
                "plug": {"snap": "bitwarden", "plug": "opengl"},
                "interface": "opengl",
            },
            {
                "slot": {"snap": "core", "slot": "pulseaudio"},
                "plug": {"snap": "bitwarden", "plug": "pulseaudio"},
                "interface": "pulseaudio",
            },
            {
                "slot": {"snap": "core", "slot": "unity7"},
                "plug": {"snap": "bitwarden", "plug": "unity7"},
                "interface": "unity7",
            },
            {
                "slot": {"snap": "core", "slot": "wayland"},
                "plug": {"snap": "bitwarden", "plug": "wayland"},
                "interface": "wayland",
            },
            {
                "slot": {"snap": "core", "slot": "x11"},
                "plug": {"snap": "bitwarden", "plug": "x11"},
                "interface": "x11",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bw", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "network-bind"},
                "plug": {"snap": "bw", "plug": "network-bind"},
                "interface": "network-bind",
            },
        ],
        "plugs": [
            {
                "snap": "bitwarden",
                "plug": "audio-playback",
                "interface": "audio-playback",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "audio-playback"}],
            },
            {
                "snap": "bitwarden",
                "plug": "browser-support",
                "interface": "browser-support",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "browser-support"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop",
                "interface": "desktop",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop-legacy",
                "interface": "desktop-legacy",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop-legacy"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gnome-3-28-1804",
                "interface": "content",
                "attrs": {
                    "content": "gnome-3-28-1804",
                    "default-provider": "gnome-3-28-1804",
                    "target": "$SNAP/gnome-platform",
                },
                "apps": ["bitwarden"],
                "connections": [{"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gsettings",
                "interface": "gsettings",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "gsettings"}],
            },
            {
                "snap": "bitwarden",
                "plug": "home",
                "interface": "home",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "home"}],
            },
            {
                "snap": "bitwarden",
                "plug": "network",
                "interface": "network",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bitwarden",
                "plug": "opengl",
                "interface": "opengl",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "opengl"}],
            },
            {
                "snap": "bitwarden",
                "plug": "pulseaudio",
                "interface": "pulseaudio",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "pulseaudio"}],
            },
            {
                "snap": "bitwarden",
                "plug": "unity7",
                "interface": "unity7",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "unity7"}],
            },
            {
                "snap": "bitwarden",
                "plug": "wayland",
                "interface": "wayland",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "wayland"}],
            },
            {
                "snap": "bitwarden",
                "plug": "x11",
                "interface": "x11",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "x11"}],
            },
            {
                "snap": "bw",
                "plug": "network",
                "interface": "network",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bw",
                "plug": "network-bind",
                "interface": "network-bind",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network-bind"}],
            },
        ],
        "slots": [
            {
                "snap": "core",
                "slot": "audio-playback",
                "interface": "audio-playback",
                "connections": [{"snap": "bitwarden", "plug": "audio-playback"}],
            },
            {
                "snap": "core",
                "slot": "browser-support",
                "interface": "browser-support",
                "connections": [{"snap": "bitwarden", "plug": "browser-support"}],
            },
            {
                "snap": "core",
                "slot": "desktop",
                "interface": "desktop",
                "connections": [{"snap": "bitwarden", "plug": "desktop"}],
            },
            {
                "snap": "core",
                "slot": "desktop-legacy",
                "interface": "desktop-legacy",
                "connections": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            },
            {
                "snap": "core",
                "slot": "gsettings",
                "interface": "gsettings",
                "connections": [{"snap": "bitwarden", "plug": "gsettings"}],
            },
            {
                "snap": "core",
                "slot": "home",
                "interface": "home",
                "connections": [{"snap": "bitwarden", "plug": "home"}],
            },
            {
                "snap": "core",
                "slot": "network",
                "interface": "network",
                "connections": [
                    {"snap": "bitwarden", "plug": "network"},
                    {"snap": "bw", "plug": "network"},
                ],
            },
            {
                "snap": "core",
                "slot": "network-bind",
                "interface": "network-bind",
                "connections": [{"snap": "bw", "plug": "network-bind"}],
            },
            {
                "snap": "core",
                "slot": "opengl",
                "interface": "opengl",
                "connections": [{"snap": "bitwarden", "plug": "opengl"}],
            },
            {
                "snap": "core",
                "slot": "pulseaudio",
                "interface": "pulseaudio",
                "connections": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            },
            {
                "snap": "core",
                "slot": "unity7",
                "interface": "unity7",
                "connections": [{"snap": "bitwarden", "plug": "unity7"}],
            },
            {
                "snap": "core",
                "slot": "wayland",
                "interface": "wayland",
                "connections": [{"snap": "bitwarden", "plug": "wayland"}],
            },
            {
                "snap": "core",
                "slot": "x11",
                "interface": "x11",
                "connections": [{"snap": "bitwarden", "plug": "x11"}],
            },
            {
                "snap": "gnome-3-28-1804",
                "slot": "gnome-3-28-1804",
                "interface": "content",
                "attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                "connections": [{"snap": "bitwarden", "plug": "gnome-3-28-1804"}],
            },
        ],
    }


@pytest.fixture
def snap_connections_all():
    return {
        "established": [
            {
                "slot": {"snap": "core", "slot": "audio-playback"},
                "plug": {"snap": "bitwarden", "plug": "audio-playback"},
                "interface": "audio-playback",
            },
            {
                "slot": {"snap": "core", "slot": "browser-support"},
                "plug": {"snap": "bitwarden", "plug": "browser-support"},
                "interface": "browser-support",
            },
            {
                "slot": {"snap": "core", "slot": "desktop"},
                "plug": {"snap": "bitwarden", "plug": "desktop"},
                "interface": "desktop",
            },
            {
                "slot": {"snap": "core", "slot": "desktop-legacy"},
                "plug": {"snap": "bitwarden", "plug": "desktop-legacy"},
                "interface": "desktop-legacy",
            },
            {
                "slot": {"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"},
                "plug": {"snap": "bitwarden", "plug": "gnome-3-28-1804"},
                "interface": "content",
                "slot-attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                "plug-attrs": {
                    "content": "gnome-3-28-1804",
                    "default-provider": "gnome-3-28-1804",
                    "target": "$SNAP/gnome-platform",
                },
            },
            {
                "slot": {"snap": "core", "slot": "gsettings"},
                "plug": {"snap": "bitwarden", "plug": "gsettings"},
                "interface": "gsettings",
            },
            {
                "slot": {"snap": "core", "slot": "home"},
                "plug": {"snap": "bitwarden", "plug": "home"},
                "interface": "home",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bitwarden", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "opengl"},
                "plug": {"snap": "bitwarden", "plug": "opengl"},
                "interface": "opengl",
            },
            {
                "slot": {"snap": "core", "slot": "pulseaudio"},
                "plug": {"snap": "bitwarden", "plug": "pulseaudio"},
                "interface": "pulseaudio",
            },
            {
                "slot": {"snap": "core", "slot": "unity7"},
                "plug": {"snap": "bitwarden", "plug": "unity7"},
                "interface": "unity7",
            },
            {
                "slot": {"snap": "core", "slot": "wayland"},
                "plug": {"snap": "bitwarden", "plug": "wayland"},
                "interface": "wayland",
            },
            {
                "slot": {"snap": "core", "slot": "x11"},
                "plug": {"snap": "bitwarden", "plug": "x11"},
                "interface": "x11",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bw", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "network-bind"},
                "plug": {"snap": "bw", "plug": "network-bind"},
                "interface": "network-bind",
            },
        ],
        "plugs": [
            {
                "snap": "bitwarden",
                "plug": "audio-playback",
                "interface": "audio-playback",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "audio-playback"}],
            },
            {
                "snap": "bitwarden",
                "plug": "browser-support",
                "interface": "browser-support",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "browser-support"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop",
                "interface": "desktop",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop-legacy",
                "interface": "desktop-legacy",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop-legacy"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gnome-3-28-1804",
                "interface": "content",
                "attrs": {
                    "content": "gnome-3-28-1804",
                    "default-provider": "gnome-3-28-1804",
                    "target": "$SNAP/gnome-platform",
                },
                "apps": ["bitwarden"],
                "connections": [{"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gsettings",
                "interface": "gsettings",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "gsettings"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gtk-3-themes",
                "interface": "content",
                "attrs": {
                    "content": "gtk-3-themes",
                    "default-provider": "gtk-common-themes",
                    "target": "$SNAP/data-dir/themes",
                },
                "apps": ["bitwarden"],
            },
            {
                "snap": "bitwarden",
                "plug": "home",
                "interface": "home",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "home"}],
            },
            {
                "snap": "bitwarden",
                "plug": "icon-themes",
                "interface": "content",
                "attrs": {
                    "content": "icon-themes",
                    "default-provider": "gtk-common-themes",
                    "target": "$SNAP/data-dir/icons",
                },
                "apps": ["bitwarden"],
            },
            {
                "snap": "bitwarden",
                "plug": "network",
                "interface": "network",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bitwarden",
                "plug": "opengl",
                "interface": "opengl",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "opengl"}],
            },
            {
                "snap": "bitwarden",
                "plug": "password-manager-service",
                "interface": "password-manager-service",
                "apps": ["bitwarden"],
            },
            {
                "snap": "bitwarden",
                "plug": "pulseaudio",
                "interface": "pulseaudio",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "pulseaudio"}],
            },
            {
                "snap": "bitwarden",
                "plug": "sound-themes",
                "interface": "content",
                "attrs": {
                    "content": "sound-themes",
                    "default-provider": "gtk-common-themes",
                    "target": "$SNAP/data-dir/sounds",
                },
                "apps": ["bitwarden"],
            },
            {
                "snap": "bitwarden",
                "plug": "unity7",
                "interface": "unity7",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "unity7"}],
            },
            {
                "snap": "bitwarden",
                "plug": "wayland",
                "interface": "wayland",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "wayland"}],
            },
            {
                "snap": "bitwarden",
                "plug": "x11",
                "interface": "x11",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "x11"}],
            },
            {
                "snap": "bw",
                "plug": "network",
                "interface": "network",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bw",
                "plug": "network-bind",
                "interface": "network-bind",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network-bind"}],
            },
        ],
        "slots": [
            {"snap": "core", "slot": "account-control", "interface": "account-control"},
            {"snap": "core", "slot": "accounts-service", "interface": "accounts-service"},
            {"snap": "core", "slot": "acrn-support", "interface": "acrn-support"},
            {"snap": "core", "slot": "adb-support", "interface": "adb-support"},
            {"snap": "core", "slot": "allegro-vcu", "interface": "allegro-vcu"},
            {"snap": "core", "slot": "alsa", "interface": "alsa"},
            {"snap": "core", "slot": "appstream-metadata", "interface": "appstream-metadata"},
            {
                "snap": "core",
                "slot": "audio-playback",
                "interface": "audio-playback",
                "connections": [{"snap": "bitwarden", "plug": "audio-playback"}],
            },
            {"snap": "core", "slot": "audio-record", "interface": "audio-record"},
            {
                "snap": "core",
                "slot": "autopilot-introspection",
                "interface": "autopilot-introspection",
            },
            {"snap": "core", "slot": "avahi-control", "interface": "avahi-control"},
            {"snap": "core", "slot": "avahi-observe", "interface": "avahi-observe"},
            {"snap": "core", "slot": "block-devices", "interface": "block-devices"},
            {"snap": "core", "slot": "bluetooth-control", "interface": "bluetooth-control"},
            {"snap": "core", "slot": "bluez", "interface": "bluez"},
            {"snap": "core", "slot": "broadcom-asic-control", "interface": "broadcom-asic-control"},
            {
                "snap": "core",
                "slot": "browser-support",
                "interface": "browser-support",
                "connections": [{"snap": "bitwarden", "plug": "browser-support"}],
            },
            {"snap": "core", "slot": "calendar-service", "interface": "calendar-service"},
            {"snap": "core", "slot": "camera", "interface": "camera"},
            {"snap": "core", "slot": "can-bus", "interface": "can-bus"},
            {"snap": "core", "slot": "cifs-mount", "interface": "cifs-mount"},
            {"snap": "core", "slot": "classic-support", "interface": "classic-support"},
            {"snap": "core", "slot": "contacts-service", "interface": "contacts-service"},
            {"snap": "core", "slot": "core-support", "interface": "core-support"},
            {"snap": "core", "slot": "cpu-control", "interface": "cpu-control"},
            {"snap": "core", "slot": "cups-control", "interface": "cups-control"},
            {"snap": "core", "slot": "daemon-notify", "interface": "daemon-notify"},
            {"snap": "core", "slot": "dcdbas-control", "interface": "dcdbas-control"},
            {
                "snap": "core",
                "slot": "desktop",
                "interface": "desktop",
                "connections": [{"snap": "bitwarden", "plug": "desktop"}],
            },
            {"snap": "core", "slot": "desktop-launch", "interface": "desktop-launch"},
            {
                "snap": "core",
                "slot": "desktop-legacy",
                "interface": "desktop-legacy",
                "connections": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            },
            {"snap": "core", "slot": "device-buttons", "interface": "device-buttons"},
            {"snap": "core", "slot": "display-control", "interface": "display-control"},
            {"snap": "core", "slot": "dm-crypt", "interface": "dm-crypt"},
            {"snap": "core", "slot": "docker-support", "interface": "docker-support"},
            {"snap": "core", "slot": "dvb", "interface": "dvb"},
            {"snap": "core", "slot": "firewall-control", "interface": "firewall-control"},
            {"snap": "core", "slot": "fpga", "interface": "fpga"},
            {"snap": "core", "slot": "framebuffer", "interface": "framebuffer"},
            {"snap": "core", "slot": "fuse-support", "interface": "fuse-support"},
            {"snap": "core", "slot": "fwupd", "interface": "fwupd"},
            {"snap": "core", "slot": "gconf", "interface": "gconf"},
            {"snap": "core", "slot": "gpg-keys", "interface": "gpg-keys"},
            {"snap": "core", "slot": "gpg-public-keys", "interface": "gpg-public-keys"},
            {"snap": "core", "slot": "gpio-control", "interface": "gpio-control"},
            {"snap": "core", "slot": "gpio-memory-control", "interface": "gpio-memory-control"},
            {"snap": "core", "slot": "greengrass-support", "interface": "greengrass-support"},
            {
                "snap": "core",
                "slot": "gsettings",
                "interface": "gsettings",
                "connections": [{"snap": "bitwarden", "plug": "gsettings"}],
            },
            {"snap": "core", "slot": "hardware-observe", "interface": "hardware-observe"},
            {
                "snap": "core",
                "slot": "hardware-random-control",
                "interface": "hardware-random-control",
            },
            {
                "snap": "core",
                "slot": "hardware-random-observe",
                "interface": "hardware-random-observe",
            },
            {
                "snap": "core",
                "slot": "home",
                "interface": "home",
                "connections": [{"snap": "bitwarden", "plug": "home"}],
            },
            {"snap": "core", "slot": "hostname-control", "interface": "hostname-control"},
            {"snap": "core", "slot": "hugepages-control", "interface": "hugepages-control"},
            {"snap": "core", "slot": "intel-mei", "interface": "intel-mei"},
            {"snap": "core", "slot": "io-ports-control", "interface": "io-ports-control"},
            {"snap": "core", "slot": "ion-memory-control", "interface": "ion-memory-control"},
            {"snap": "core", "slot": "jack1", "interface": "jack1"},
            {"snap": "core", "slot": "joystick", "interface": "joystick"},
            {"snap": "core", "slot": "juju-client-observe", "interface": "juju-client-observe"},
            {"snap": "core", "slot": "kernel-crypto-api", "interface": "kernel-crypto-api"},
            {"snap": "core", "slot": "kernel-module-control", "interface": "kernel-module-control"},
            {"snap": "core", "slot": "kernel-module-load", "interface": "kernel-module-load"},
            {"snap": "core", "slot": "kernel-module-observe", "interface": "kernel-module-observe"},
            {"snap": "core", "slot": "kubernetes-support", "interface": "kubernetes-support"},
            {"snap": "core", "slot": "kvm", "interface": "kvm"},
            {"snap": "core", "slot": "libvirt", "interface": "libvirt"},
            {"snap": "core", "slot": "locale-control", "interface": "locale-control"},
            {"snap": "core", "slot": "log-observe", "interface": "log-observe"},
            {"snap": "core", "slot": "login-session-control", "interface": "login-session-control"},
            {"snap": "core", "slot": "login-session-observe", "interface": "login-session-observe"},
            {"snap": "core", "slot": "lxd-support", "interface": "lxd-support"},
            {"snap": "core", "slot": "media-control", "interface": "media-control"},
            {"snap": "core", "slot": "microstack-support", "interface": "microstack-support"},
            {"snap": "core", "slot": "modem-manager", "interface": "modem-manager"},
            {"snap": "core", "slot": "mount-control", "interface": "mount-control"},
            {"snap": "core", "slot": "mount-observe", "interface": "mount-observe"},
            {"snap": "core", "slot": "multipass-support", "interface": "multipass-support"},
            {"snap": "core", "slot": "netlink-audit", "interface": "netlink-audit"},
            {"snap": "core", "slot": "netlink-connector", "interface": "netlink-connector"},
            {
                "snap": "core",
                "slot": "network",
                "interface": "network",
                "connections": [
                    {"snap": "bitwarden", "plug": "network"},
                    {"snap": "bw", "plug": "network"},
                ],
            },
            {
                "snap": "core",
                "slot": "network-bind",
                "interface": "network-bind",
                "connections": [{"snap": "bw", "plug": "network-bind"}],
            },
            {"snap": "core", "slot": "network-control", "interface": "network-control"},
            {"snap": "core", "slot": "network-manager", "interface": "network-manager"},
            {
                "snap": "core",
                "slot": "network-manager-observe",
                "interface": "network-manager-observe",
            },
            {"snap": "core", "slot": "network-observe", "interface": "network-observe"},
            {"snap": "core", "slot": "network-setup-control", "interface": "network-setup-control"},
            {"snap": "core", "slot": "network-setup-observe", "interface": "network-setup-observe"},
            {"snap": "core", "slot": "network-status", "interface": "network-status"},
            {
                "snap": "core",
                "slot": "nvidia-drivers-support",
                "interface": "nvidia-drivers-support",
            },
            {"snap": "core", "slot": "ofono", "interface": "ofono"},
            {
                "snap": "core",
                "slot": "opengl",
                "interface": "opengl",
                "connections": [{"snap": "bitwarden", "plug": "opengl"}],
            },
            {"snap": "core", "slot": "openvswitch", "interface": "openvswitch"},
            {"snap": "core", "slot": "openvswitch-support", "interface": "openvswitch-support"},
            {"snap": "core", "slot": "optical-drive", "interface": "optical-drive"},
            {"snap": "core", "slot": "packagekit-control", "interface": "packagekit-control"},
            {
                "snap": "core",
                "slot": "password-manager-service",
                "interface": "password-manager-service",
            },
            {"snap": "core", "slot": "pcscd", "interface": "pcscd"},
            {"snap": "core", "slot": "personal-files", "interface": "personal-files"},
            {
                "snap": "core",
                "slot": "physical-memory-control",
                "interface": "physical-memory-control",
            },
            {
                "snap": "core",
                "slot": "physical-memory-observe",
                "interface": "physical-memory-observe",
            },
            {"snap": "core", "slot": "polkit", "interface": "polkit"},
            {"snap": "core", "slot": "power-control", "interface": "power-control"},
            {"snap": "core", "slot": "ppp", "interface": "ppp"},
            {"snap": "core", "slot": "process-control", "interface": "process-control"},
            {"snap": "core", "slot": "ptp", "interface": "ptp"},
            {
                "snap": "core",
                "slot": "pulseaudio",
                "interface": "pulseaudio",
                "connections": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            },
            {"snap": "core", "slot": "qualcomm-ipc-router", "interface": "qualcomm-ipc-router"},
            {"snap": "core", "slot": "raw-input", "interface": "raw-input"},
            {"snap": "core", "slot": "raw-usb", "interface": "raw-usb"},
            {"snap": "core", "slot": "removable-media", "interface": "removable-media"},
            {
                "snap": "core",
                "slot": "screen-inhibit-control",
                "interface": "screen-inhibit-control",
            },
            {"snap": "core", "slot": "screencast-legacy", "interface": "screencast-legacy"},
            {"snap": "core", "slot": "scsi-generic", "interface": "scsi-generic"},
            {"snap": "core", "slot": "sd-control", "interface": "sd-control"},
            {"snap": "core", "slot": "shared-memory", "interface": "shared-memory"},
            {"snap": "core", "slot": "shutdown", "interface": "shutdown"},
            {"snap": "core", "slot": "snap-refresh-control", "interface": "snap-refresh-control"},
            {"snap": "core", "slot": "snap-themes-control", "interface": "snap-themes-control"},
            {"snap": "core", "slot": "snapd-control", "interface": "snapd-control"},
            {"snap": "core", "slot": "ssh-keys", "interface": "ssh-keys"},
            {"snap": "core", "slot": "ssh-public-keys", "interface": "ssh-public-keys"},
            {"snap": "core", "slot": "steam-support", "interface": "steam-support"},
            {"snap": "core", "slot": "system-backup", "interface": "system-backup"},
            {"snap": "core", "slot": "system-files", "interface": "system-files"},
            {"snap": "core", "slot": "system-observe", "interface": "system-observe"},
            {"snap": "core", "slot": "system-packages-doc", "interface": "system-packages-doc"},
            {"snap": "core", "slot": "system-source-code", "interface": "system-source-code"},
            {"snap": "core", "slot": "system-trace", "interface": "system-trace"},
            {"snap": "core", "slot": "tee", "interface": "tee"},
            {"snap": "core", "slot": "time-control", "interface": "time-control"},
            {"snap": "core", "slot": "timeserver-control", "interface": "timeserver-control"},
            {"snap": "core", "slot": "timezone-control", "interface": "timezone-control"},
            {"snap": "core", "slot": "tpm", "interface": "tpm"},
            {"snap": "core", "slot": "u2f-devices", "interface": "u2f-devices"},
            {"snap": "core", "slot": "udisks2", "interface": "udisks2"},
            {"snap": "core", "slot": "uhid", "interface": "uhid"},
            {"snap": "core", "slot": "uinput", "interface": "uinput"},
            {
                "snap": "core",
                "slot": "unity7",
                "interface": "unity7",
                "connections": [{"snap": "bitwarden", "plug": "unity7"}],
            },
            {"snap": "core", "slot": "upower-observe", "interface": "upower-observe"},
            {"snap": "core", "slot": "userns", "interface": "userns"},
            {"snap": "core", "slot": "vcio", "interface": "vcio"},
            {
                "snap": "core",
                "slot": "wayland",
                "interface": "wayland",
                "connections": [{"snap": "bitwarden", "plug": "wayland"}],
            },
            {
                "snap": "core",
                "slot": "x11",
                "interface": "x11",
                "connections": [{"snap": "bitwarden", "plug": "x11"}],
            },
            {"snap": "core", "slot": "xilinx-dma", "interface": "xilinx-dma"},
            {
                "snap": "gnome-3-28-1804",
                "slot": "gnome-3-28-1804",
                "interface": "content",
                "attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                "connections": [{"snap": "bitwarden", "plug": "gnome-3-28-1804"}],
            },
            {
                "snap": "gnome-3-34-1804",
                "slot": "gnome-3-34-1804",
                "interface": "content",
                "attrs": {"content": "gnome-3-34-1804", "read": ["/"]},
            },
        ],
    }


@pytest.fixture
def snap_connections_name():
    return {
        "established": [
            {
                "slot": {"snap": "core", "slot": "audio-playback"},
                "plug": {"snap": "bitwarden", "plug": "audio-playback"},
                "interface": "audio-playback",
            },
            {
                "slot": {"snap": "core", "slot": "browser-support"},
                "plug": {"snap": "bitwarden", "plug": "browser-support"},
                "interface": "browser-support",
            },
            {
                "slot": {"snap": "core", "slot": "desktop"},
                "plug": {"snap": "bitwarden", "plug": "desktop"},
                "interface": "desktop",
            },
            {
                "slot": {"snap": "core", "slot": "desktop-legacy"},
                "plug": {"snap": "bitwarden", "plug": "desktop-legacy"},
                "interface": "desktop-legacy",
            },
            {
                "slot": {"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"},
                "plug": {"snap": "bitwarden", "plug": "gnome-3-28-1804"},
                "interface": "content",
                "slot-attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                "plug-attrs": {
                    "content": "gnome-3-28-1804",
                    "default-provider": "gnome-3-28-1804",
                    "target": "$SNAP/gnome-platform",
                },
            },
            {
                "slot": {"snap": "core", "slot": "gsettings"},
                "plug": {"snap": "bitwarden", "plug": "gsettings"},
                "interface": "gsettings",
            },
            {
                "slot": {"snap": "core", "slot": "home"},
                "plug": {"snap": "bitwarden", "plug": "home"},
                "interface": "home",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bitwarden", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "opengl"},
                "plug": {"snap": "bitwarden", "plug": "opengl"},
                "interface": "opengl",
            },
            {
                "slot": {"snap": "core", "slot": "pulseaudio"},
                "plug": {"snap": "bitwarden", "plug": "pulseaudio"},
                "interface": "pulseaudio",
            },
            {
                "slot": {"snap": "core", "slot": "unity7"},
                "plug": {"snap": "bitwarden", "plug": "unity7"},
                "interface": "unity7",
            },
            {
                "slot": {"snap": "core", "slot": "wayland"},
                "plug": {"snap": "bitwarden", "plug": "wayland"},
                "interface": "wayland",
            },
            {
                "slot": {"snap": "core", "slot": "x11"},
                "plug": {"snap": "bitwarden", "plug": "x11"},
                "interface": "x11",
            },
        ],
        "plugs": [
            {
                "snap": "bitwarden",
                "plug": "audio-playback",
                "interface": "audio-playback",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "audio-playback"}],
            },
            {
                "snap": "bitwarden",
                "plug": "browser-support",
                "interface": "browser-support",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "browser-support"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop",
                "interface": "desktop",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop-legacy",
                "interface": "desktop-legacy",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop-legacy"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gnome-3-28-1804",
                "interface": "content",
                "attrs": {
                    "content": "gnome-3-28-1804",
                    "default-provider": "gnome-3-28-1804",
                    "target": "$SNAP/gnome-platform",
                },
                "apps": ["bitwarden"],
                "connections": [{"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gsettings",
                "interface": "gsettings",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "gsettings"}],
            },
            {
                "snap": "bitwarden",
                "plug": "home",
                "interface": "home",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "home"}],
            },
            {
                "snap": "bitwarden",
                "plug": "network",
                "interface": "network",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bitwarden",
                "plug": "opengl",
                "interface": "opengl",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "opengl"}],
            },
            {
                "snap": "bitwarden",
                "plug": "pulseaudio",
                "interface": "pulseaudio",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "pulseaudio"}],
            },
            {
                "snap": "bitwarden",
                "plug": "unity7",
                "interface": "unity7",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "unity7"}],
            },
            {
                "snap": "bitwarden",
                "plug": "wayland",
                "interface": "wayland",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "wayland"}],
            },
            {
                "snap": "bitwarden",
                "plug": "x11",
                "interface": "x11",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "x11"}],
            },
        ],
        "slots": [
            {
                "snap": "core",
                "slot": "audio-playback",
                "interface": "audio-playback",
                "connections": [{"snap": "bitwarden", "plug": "audio-playback"}],
            },
            {
                "snap": "core",
                "slot": "browser-support",
                "interface": "browser-support",
                "connections": [{"snap": "bitwarden", "plug": "browser-support"}],
            },
            {
                "snap": "core",
                "slot": "desktop",
                "interface": "desktop",
                "connections": [{"snap": "bitwarden", "plug": "desktop"}],
            },
            {
                "snap": "core",
                "slot": "desktop-legacy",
                "interface": "desktop-legacy",
                "connections": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            },
            {
                "snap": "core",
                "slot": "gsettings",
                "interface": "gsettings",
                "connections": [{"snap": "bitwarden", "plug": "gsettings"}],
            },
            {
                "snap": "core",
                "slot": "home",
                "interface": "home",
                "connections": [{"snap": "bitwarden", "plug": "home"}],
            },
            {
                "snap": "core",
                "slot": "network",
                "interface": "network",
                "connections": [{"snap": "bitwarden", "plug": "network"}],
            },
            {
                "snap": "core",
                "slot": "opengl",
                "interface": "opengl",
                "connections": [{"snap": "bitwarden", "plug": "opengl"}],
            },
            {
                "snap": "core",
                "slot": "pulseaudio",
                "interface": "pulseaudio",
                "connections": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            },
            {
                "snap": "core",
                "slot": "unity7",
                "interface": "unity7",
                "connections": [{"snap": "bitwarden", "plug": "unity7"}],
            },
            {
                "snap": "core",
                "slot": "wayland",
                "interface": "wayland",
                "connections": [{"snap": "bitwarden", "plug": "wayland"}],
            },
            {
                "snap": "core",
                "slot": "x11",
                "interface": "x11",
                "connections": [{"snap": "bitwarden", "plug": "x11"}],
            },
            {
                "snap": "gnome-3-28-1804",
                "slot": "gnome-3-28-1804",
                "interface": "content",
                "attrs": {"content": "gnome-3-28-1804", "read": ["/"]},
                "connections": [{"snap": "bitwarden", "plug": "gnome-3-28-1804"}],
            },
        ],
    }


@pytest.fixture
def snap_connections_core():
    return {
        "established": [
            {
                "slot": {"snap": "core", "slot": "audio-playback"},
                "plug": {"snap": "bitwarden", "plug": "audio-playback"},
                "interface": "audio-playback",
            },
            {
                "slot": {"snap": "core", "slot": "browser-support"},
                "plug": {"snap": "bitwarden", "plug": "browser-support"},
                "interface": "browser-support",
            },
            {
                "slot": {"snap": "core", "slot": "desktop"},
                "plug": {"snap": "bitwarden", "plug": "desktop"},
                "interface": "desktop",
            },
            {
                "slot": {"snap": "core", "slot": "desktop-legacy"},
                "plug": {"snap": "bitwarden", "plug": "desktop-legacy"},
                "interface": "desktop-legacy",
            },
            {
                "slot": {"snap": "core", "slot": "gsettings"},
                "plug": {"snap": "bitwarden", "plug": "gsettings"},
                "interface": "gsettings",
            },
            {
                "slot": {"snap": "core", "slot": "home"},
                "plug": {"snap": "bitwarden", "plug": "home"},
                "interface": "home",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bitwarden", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "opengl"},
                "plug": {"snap": "bitwarden", "plug": "opengl"},
                "interface": "opengl",
            },
            {
                "slot": {"snap": "core", "slot": "pulseaudio"},
                "plug": {"snap": "bitwarden", "plug": "pulseaudio"},
                "interface": "pulseaudio",
            },
            {
                "slot": {"snap": "core", "slot": "unity7"},
                "plug": {"snap": "bitwarden", "plug": "unity7"},
                "interface": "unity7",
            },
            {
                "slot": {"snap": "core", "slot": "wayland"},
                "plug": {"snap": "bitwarden", "plug": "wayland"},
                "interface": "wayland",
            },
            {
                "slot": {"snap": "core", "slot": "x11"},
                "plug": {"snap": "bitwarden", "plug": "x11"},
                "interface": "x11",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bw", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "network-bind"},
                "plug": {"snap": "bw", "plug": "network-bind"},
                "interface": "network-bind",
            },
        ],
        "plugs": [
            {
                "snap": "bitwarden",
                "plug": "audio-playback",
                "interface": "audio-playback",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "audio-playback"}],
            },
            {
                "snap": "bitwarden",
                "plug": "browser-support",
                "interface": "browser-support",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "browser-support"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop",
                "interface": "desktop",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop"}],
            },
            {
                "snap": "bitwarden",
                "plug": "desktop-legacy",
                "interface": "desktop-legacy",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "desktop-legacy"}],
            },
            {
                "snap": "bitwarden",
                "plug": "gsettings",
                "interface": "gsettings",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "gsettings"}],
            },
            {
                "snap": "bitwarden",
                "plug": "home",
                "interface": "home",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "home"}],
            },
            {
                "snap": "bitwarden",
                "plug": "network",
                "interface": "network",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bitwarden",
                "plug": "opengl",
                "interface": "opengl",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "opengl"}],
            },
            {
                "snap": "bitwarden",
                "plug": "pulseaudio",
                "interface": "pulseaudio",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "pulseaudio"}],
            },
            {
                "snap": "bitwarden",
                "plug": "unity7",
                "interface": "unity7",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "unity7"}],
            },
            {
                "snap": "bitwarden",
                "plug": "wayland",
                "interface": "wayland",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "wayland"}],
            },
            {
                "snap": "bitwarden",
                "plug": "x11",
                "interface": "x11",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "x11"}],
            },
            {
                "snap": "bw",
                "plug": "network",
                "interface": "network",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bw",
                "plug": "network-bind",
                "interface": "network-bind",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network-bind"}],
            },
        ],
        "slots": [
            {
                "snap": "core",
                "slot": "audio-playback",
                "interface": "audio-playback",
                "connections": [{"snap": "bitwarden", "plug": "audio-playback"}],
            },
            {
                "snap": "core",
                "slot": "browser-support",
                "interface": "browser-support",
                "connections": [{"snap": "bitwarden", "plug": "browser-support"}],
            },
            {
                "snap": "core",
                "slot": "desktop",
                "interface": "desktop",
                "connections": [{"snap": "bitwarden", "plug": "desktop"}],
            },
            {
                "snap": "core",
                "slot": "desktop-legacy",
                "interface": "desktop-legacy",
                "connections": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            },
            {
                "snap": "core",
                "slot": "gsettings",
                "interface": "gsettings",
                "connections": [{"snap": "bitwarden", "plug": "gsettings"}],
            },
            {
                "snap": "core",
                "slot": "home",
                "interface": "home",
                "connections": [{"snap": "bitwarden", "plug": "home"}],
            },
            {
                "snap": "core",
                "slot": "network",
                "interface": "network",
                "connections": [
                    {"snap": "bitwarden", "plug": "network"},
                    {"snap": "bw", "plug": "network"},
                ],
            },
            {
                "snap": "core",
                "slot": "network-bind",
                "interface": "network-bind",
                "connections": [{"snap": "bw", "plug": "network-bind"}],
            },
            {
                "snap": "core",
                "slot": "opengl",
                "interface": "opengl",
                "connections": [{"snap": "bitwarden", "plug": "opengl"}],
            },
            {
                "snap": "core",
                "slot": "pulseaudio",
                "interface": "pulseaudio",
                "connections": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            },
            {
                "snap": "core",
                "slot": "unity7",
                "interface": "unity7",
                "connections": [{"snap": "bitwarden", "plug": "unity7"}],
            },
            {
                "snap": "core",
                "slot": "wayland",
                "interface": "wayland",
                "connections": [{"snap": "bitwarden", "plug": "wayland"}],
            },
            {
                "snap": "core",
                "slot": "x11",
                "interface": "x11",
                "connections": [{"snap": "bitwarden", "plug": "x11"}],
            },
        ],
    }


@pytest.fixture
def snap_connections_interface():
    return {
        "established": [
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bitwarden", "plug": "network"},
                "interface": "network",
            },
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bw", "plug": "network"},
                "interface": "network",
            },
        ],
        "plugs": [
            {
                "snap": "bitwarden",
                "plug": "network",
                "interface": "network",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
            {
                "snap": "bw",
                "plug": "network",
                "interface": "network",
                "apps": ["bw"],
                "connections": [{"snap": "core", "slot": "network"}],
            },
        ],
        "slots": [
            {
                "snap": "core",
                "slot": "network",
                "interface": "network",
                "connections": [
                    {"snap": "bitwarden", "plug": "network"},
                    {"snap": "bw", "plug": "network"},
                ],
            }
        ],
    }


@pytest.fixture
def snap_connections_interface_name():
    return {
        "established": [
            {
                "slot": {"snap": "core", "slot": "network"},
                "plug": {"snap": "bitwarden", "plug": "network"},
                "interface": "network",
            }
        ],
        "plugs": [
            {
                "snap": "bitwarden",
                "plug": "network",
                "interface": "network",
                "apps": ["bitwarden"],
                "connections": [{"snap": "core", "slot": "network"}],
            }
        ],
        "slots": [
            {
                "snap": "core",
                "slot": "network",
                "interface": "network",
                "connections": [{"snap": "bitwarden", "plug": "network"}],
            }
        ],
    }


@pytest.fixture
def snap_plugs():
    return {
        "audio-playback": {
            "interface": "audio-playback",
            "connections": [{"snap": "core", "slot": "audio-playback"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "browser-support": {
            "interface": "browser-support",
            "connections": [{"snap": "core", "slot": "browser-support"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "desktop": {
            "interface": "desktop",
            "connections": [{"snap": "core", "slot": "desktop"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "desktop-legacy": {
            "interface": "desktop-legacy",
            "connections": [{"snap": "core", "slot": "desktop-legacy"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "gnome-3-28-1804": {
            "interface": "content",
            "connections": [{"snap": "gnome-3-28-1804", "slot": "gnome-3-28-1804"}],
            "snap": "bitwarden",
            "attrs": {
                "content": "gnome-3-28-1804",
                "default-provider": "gnome-3-28-1804",
                "target": "$SNAP/gnome-platform",
            },
        },
        "gsettings": {
            "interface": "gsettings",
            "connections": [{"snap": "core", "slot": "gsettings"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "gtk-3-themes": {
            "interface": "content",
            "connections": [],
            "snap": "bitwarden",
            "attrs": {
                "content": "gtk-3-themes",
                "default-provider": "gtk-common-themes",
                "target": "$SNAP/data-dir/themes",
            },
        },
        "home": {
            "interface": "home",
            "connections": [{"snap": "core", "slot": "home"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "icon-themes": {
            "interface": "content",
            "connections": [],
            "snap": "bitwarden",
            "attrs": {
                "content": "icon-themes",
                "default-provider": "gtk-common-themes",
                "target": "$SNAP/data-dir/icons",
            },
        },
        "network": {
            "interface": "network",
            "connections": [{"snap": "core", "slot": "network"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "opengl": {
            "interface": "opengl",
            "connections": [{"snap": "core", "slot": "opengl"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "password-manager-service": {
            "interface": "password-manager-service",
            "connections": [],
            "snap": "bitwarden",
            "attrs": {},
        },
        "pulseaudio": {
            "interface": "pulseaudio",
            "connections": [{"snap": "core", "slot": "pulseaudio"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "sound-themes": {
            "interface": "content",
            "connections": [],
            "snap": "bitwarden",
            "attrs": {
                "content": "sound-themes",
                "default-provider": "gtk-common-themes",
                "target": "$SNAP/data-dir/sounds",
            },
        },
        "unity7": {
            "interface": "unity7",
            "connections": [{"snap": "core", "slot": "unity7"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "wayland": {
            "interface": "wayland",
            "connections": [{"snap": "core", "slot": "wayland"}],
            "snap": "bitwarden",
            "attrs": {},
        },
        "x11": {
            "interface": "x11",
            "connections": [{"snap": "core", "slot": "x11"}],
            "snap": "bitwarden",
            "attrs": {},
        },
    }


@pytest.fixture
def snap_slots():
    return {
        "account-control": {
            "interface": "account-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "accounts-service": {
            "interface": "accounts-service",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "acrn-support": {
            "interface": "acrn-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "adb-support": {"interface": "adb-support", "connections": [], "snap": "core", "attrs": {}},
        "allegro-vcu": {"interface": "allegro-vcu", "connections": [], "snap": "core", "attrs": {}},
        "alsa": {"interface": "alsa", "connections": [], "snap": "core", "attrs": {}},
        "appstream-metadata": {
            "interface": "appstream-metadata",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "audio-playback": {
            "interface": "audio-playback",
            "connections": [{"snap": "bitwarden", "plug": "audio-playback"}],
            "snap": "core",
            "attrs": {},
        },
        "audio-record": {
            "interface": "audio-record",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "autopilot-introspection": {
            "interface": "autopilot-introspection",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "avahi-control": {
            "interface": "avahi-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "avahi-observe": {
            "interface": "avahi-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "block-devices": {
            "interface": "block-devices",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "bluetooth-control": {
            "interface": "bluetooth-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "bluez": {"interface": "bluez", "connections": [], "snap": "core", "attrs": {}},
        "broadcom-asic-control": {
            "interface": "broadcom-asic-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "browser-support": {
            "interface": "browser-support",
            "connections": [{"snap": "bitwarden", "plug": "browser-support"}],
            "snap": "core",
            "attrs": {},
        },
        "calendar-service": {
            "interface": "calendar-service",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "camera": {"interface": "camera", "connections": [], "snap": "core", "attrs": {}},
        "can-bus": {"interface": "can-bus", "connections": [], "snap": "core", "attrs": {}},
        "cifs-mount": {"interface": "cifs-mount", "connections": [], "snap": "core", "attrs": {}},
        "classic-support": {
            "interface": "classic-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "contacts-service": {
            "interface": "contacts-service",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "core-support": {
            "interface": "core-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "cpu-control": {"interface": "cpu-control", "connections": [], "snap": "core", "attrs": {}},
        "cups-control": {
            "interface": "cups-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "daemon-notify": {
            "interface": "daemon-notify",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "dcdbas-control": {
            "interface": "dcdbas-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "desktop": {
            "interface": "desktop",
            "connections": [{"snap": "bitwarden", "plug": "desktop"}],
            "snap": "core",
            "attrs": {},
        },
        "desktop-launch": {
            "interface": "desktop-launch",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "desktop-legacy": {
            "interface": "desktop-legacy",
            "connections": [{"snap": "bitwarden", "plug": "desktop-legacy"}],
            "snap": "core",
            "attrs": {},
        },
        "device-buttons": {
            "interface": "device-buttons",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "display-control": {
            "interface": "display-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "dm-crypt": {"interface": "dm-crypt", "connections": [], "snap": "core", "attrs": {}},
        "docker-support": {
            "interface": "docker-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "dvb": {"interface": "dvb", "connections": [], "snap": "core", "attrs": {}},
        "firewall-control": {
            "interface": "firewall-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "fpga": {"interface": "fpga", "connections": [], "snap": "core", "attrs": {}},
        "framebuffer": {"interface": "framebuffer", "connections": [], "snap": "core", "attrs": {}},
        "fuse-support": {
            "interface": "fuse-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "fwupd": {"interface": "fwupd", "connections": [], "snap": "core", "attrs": {}},
        "gconf": {"interface": "gconf", "connections": [], "snap": "core", "attrs": {}},
        "gpg-keys": {"interface": "gpg-keys", "connections": [], "snap": "core", "attrs": {}},
        "gpg-public-keys": {
            "interface": "gpg-public-keys",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "gpio-control": {
            "interface": "gpio-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "gpio-memory-control": {
            "interface": "gpio-memory-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "greengrass-support": {
            "interface": "greengrass-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "gsettings": {
            "interface": "gsettings",
            "connections": [{"snap": "bitwarden", "plug": "gsettings"}],
            "snap": "core",
            "attrs": {},
        },
        "hardware-observe": {
            "interface": "hardware-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "hardware-random-control": {
            "interface": "hardware-random-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "hardware-random-observe": {
            "interface": "hardware-random-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "home": {
            "interface": "home",
            "connections": [{"snap": "bitwarden", "plug": "home"}],
            "snap": "core",
            "attrs": {},
        },
        "hostname-control": {
            "interface": "hostname-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "hugepages-control": {
            "interface": "hugepages-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "intel-mei": {"interface": "intel-mei", "connections": [], "snap": "core", "attrs": {}},
        "io-ports-control": {
            "interface": "io-ports-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "ion-memory-control": {
            "interface": "ion-memory-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "jack1": {"interface": "jack1", "connections": [], "snap": "core", "attrs": {}},
        "joystick": {"interface": "joystick", "connections": [], "snap": "core", "attrs": {}},
        "juju-client-observe": {
            "interface": "juju-client-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "kernel-crypto-api": {
            "interface": "kernel-crypto-api",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "kernel-module-control": {
            "interface": "kernel-module-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "kernel-module-load": {
            "interface": "kernel-module-load",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "kernel-module-observe": {
            "interface": "kernel-module-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "kubernetes-support": {
            "interface": "kubernetes-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "kvm": {"interface": "kvm", "connections": [], "snap": "core", "attrs": {}},
        "libvirt": {"interface": "libvirt", "connections": [], "snap": "core", "attrs": {}},
        "locale-control": {
            "interface": "locale-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "log-observe": {"interface": "log-observe", "connections": [], "snap": "core", "attrs": {}},
        "login-session-control": {
            "interface": "login-session-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "login-session-observe": {
            "interface": "login-session-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "lxd-support": {"interface": "lxd-support", "connections": [], "snap": "core", "attrs": {}},
        "media-control": {
            "interface": "media-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "microstack-support": {
            "interface": "microstack-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "modem-manager": {
            "interface": "modem-manager",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "mount-control": {
            "interface": "mount-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "mount-observe": {
            "interface": "mount-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "multipass-support": {
            "interface": "multipass-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "netlink-audit": {
            "interface": "netlink-audit",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "netlink-connector": {
            "interface": "netlink-connector",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network": {
            "interface": "network",
            "connections": [
                {"snap": "bitwarden", "plug": "network"},
                {"snap": "bw", "plug": "network"},
            ],
            "snap": "core",
            "attrs": {},
        },
        "network-bind": {
            "interface": "network-bind",
            "connections": [{"snap": "bw", "plug": "network-bind"}],
            "snap": "core",
            "attrs": {},
        },
        "network-control": {
            "interface": "network-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network-manager": {
            "interface": "network-manager",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network-manager-observe": {
            "interface": "network-manager-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network-observe": {
            "interface": "network-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network-setup-control": {
            "interface": "network-setup-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network-setup-observe": {
            "interface": "network-setup-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "network-status": {
            "interface": "network-status",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "nvidia-drivers-support": {
            "interface": "nvidia-drivers-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "ofono": {"interface": "ofono", "connections": [], "snap": "core", "attrs": {}},
        "opengl": {
            "interface": "opengl",
            "connections": [{"snap": "bitwarden", "plug": "opengl"}],
            "snap": "core",
            "attrs": {},
        },
        "openvswitch": {"interface": "openvswitch", "connections": [], "snap": "core", "attrs": {}},
        "openvswitch-support": {
            "interface": "openvswitch-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "optical-drive": {
            "interface": "optical-drive",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "packagekit-control": {
            "interface": "packagekit-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "password-manager-service": {
            "interface": "password-manager-service",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "pcscd": {"interface": "pcscd", "connections": [], "snap": "core", "attrs": {}},
        "personal-files": {
            "interface": "personal-files",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "physical-memory-control": {
            "interface": "physical-memory-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "physical-memory-observe": {
            "interface": "physical-memory-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "polkit": {"interface": "polkit", "connections": [], "snap": "core", "attrs": {}},
        "power-control": {
            "interface": "power-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "ppp": {"interface": "ppp", "connections": [], "snap": "core", "attrs": {}},
        "process-control": {
            "interface": "process-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "ptp": {"interface": "ptp", "connections": [], "snap": "core", "attrs": {}},
        "pulseaudio": {
            "interface": "pulseaudio",
            "connections": [{"snap": "bitwarden", "plug": "pulseaudio"}],
            "snap": "core",
            "attrs": {},
        },
        "qualcomm-ipc-router": {
            "interface": "qualcomm-ipc-router",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "raw-input": {"interface": "raw-input", "connections": [], "snap": "core", "attrs": {}},
        "raw-usb": {"interface": "raw-usb", "connections": [], "snap": "core", "attrs": {}},
        "removable-media": {
            "interface": "removable-media",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "screen-inhibit-control": {
            "interface": "screen-inhibit-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "screencast-legacy": {
            "interface": "screencast-legacy",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "scsi-generic": {
            "interface": "scsi-generic",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "sd-control": {"interface": "sd-control", "connections": [], "snap": "core", "attrs": {}},
        "shared-memory": {
            "interface": "shared-memory",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "shutdown": {"interface": "shutdown", "connections": [], "snap": "core", "attrs": {}},
        "snap-refresh-control": {
            "interface": "snap-refresh-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "snap-themes-control": {
            "interface": "snap-themes-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "snapd-control": {
            "interface": "snapd-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "ssh-keys": {"interface": "ssh-keys", "connections": [], "snap": "core", "attrs": {}},
        "ssh-public-keys": {
            "interface": "ssh-public-keys",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "steam-support": {
            "interface": "steam-support",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "system-backup": {
            "interface": "system-backup",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "system-files": {
            "interface": "system-files",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "system-observe": {
            "interface": "system-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "system-packages-doc": {
            "interface": "system-packages-doc",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "system-source-code": {
            "interface": "system-source-code",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "system-trace": {
            "interface": "system-trace",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "tee": {"interface": "tee", "connections": [], "snap": "core", "attrs": {}},
        "time-control": {
            "interface": "time-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "timeserver-control": {
            "interface": "timeserver-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "timezone-control": {
            "interface": "timezone-control",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "tpm": {"interface": "tpm", "connections": [], "snap": "core", "attrs": {}},
        "u2f-devices": {"interface": "u2f-devices", "connections": [], "snap": "core", "attrs": {}},
        "udisks2": {"interface": "udisks2", "connections": [], "snap": "core", "attrs": {}},
        "uhid": {"interface": "uhid", "connections": [], "snap": "core", "attrs": {}},
        "uinput": {"interface": "uinput", "connections": [], "snap": "core", "attrs": {}},
        "unity7": {
            "interface": "unity7",
            "connections": [{"snap": "bitwarden", "plug": "unity7"}],
            "snap": "core",
            "attrs": {},
        },
        "upower-observe": {
            "interface": "upower-observe",
            "connections": [],
            "snap": "core",
            "attrs": {},
        },
        "userns": {"interface": "userns", "connections": [], "snap": "core", "attrs": {}},
        "vcio": {"interface": "vcio", "connections": [], "snap": "core", "attrs": {}},
        "wayland": {
            "interface": "wayland",
            "connections": [{"snap": "bitwarden", "plug": "wayland"}],
            "snap": "core",
            "attrs": {},
        },
        "x11": {
            "interface": "x11",
            "connections": [{"snap": "bitwarden", "plug": "x11"}],
            "snap": "core",
            "attrs": {},
        },
        "xilinx-dma": {"interface": "xilinx-dma", "connections": [], "snap": "core", "attrs": {}},
    }
